/*----- PROTECTED REGION ID(FastPS.h) ENABLED START -----*/
//=============================================================================
//
// file :        FastPS.h
//
// description : Include file for the FastPS class
//
// project :     CaenElsFastPS
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Elettra-Sincrotrone Trieste S.C.p.A. di interesse nazionale
// Strada Statale 14 - km 163,5 in AREA Science Park
// 34149 Basovizza, Trieste ITALY
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef FastPS_H
#define FastPS_H

#include <sstream>
#include <iostream>
#include <string>
#include <vector>
#include <stdio.h> 
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>

#include <errno.h>
#include <string.h>
#include <fcntl.h>      
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <tango.h>

#define FASTPS_DEV_TOUT     1000   /* ms. */
#define FASTPS_STATE_TOUT   3      /* sec */
#define SHORT_LEN  100
#define LONG_LEN   4000000
#define FASTPS_RECONNECT_MAXERR 5 /* sec */

#define FASTPS_MIN_WAVE 100
#define FASTPS_MAX_WAVE 500000

#define FASTPS_ON           0    // 1=on 0=off
#define FASTPS_FAULT        1    // 1=fault 0=ok
#define FASTPS_LOCAL        2    // 1=local 0=remote
#define FASTPS_CTRL         5    // 1=voltage control 0=current control
#define FASTPS_EXTREF       6    // 1=external reference 0=internal setting
#define FASTPS_RAMPING      12   // 1=ramping on 0=module is not ramping
#define FASTPS_WAVE         13   // 1=waveform running 0=waveform stopped
#define FASTPS_IOVCERR      17   // 1=input overcurrent error
#define FASTPS_CROWBARERR   18   // 1=crowbar protection intervention
#define FASTPS_OVTERR       20   // 1=over temperature 
#define FASTPS_DCLINKERR    21   // 1=DC-link undervoltage
#define FASTPS_EARTHLEAKERR 22   // 1=earth current leakage fault
#define FASTPS_EARTHFUSEERR 23   // 1=earth fuse is blown
#define FASTPS_REGERR       24   // 1=regulation fault
#define FASTPS_RIPPLEERR    25   // 1=excessive ripple
#define FASTPS_INT1ERR      26   // 1=interlock 1 tripped
#define FASTPS_INT2ERR      27   // 1=interlock 2 tripped
#define FASTPS_OVPERR       29   // 1=overpower

#define FASTPS_SOCK_OK     0
#define FASTPS_SOCK_ERR   -1
#define FASTPS_SYNTAX_ERR -2
#define FASTPS_INPUT_ERR  -3
#define FASTPS_NAK_ERR    -4

/*----- PROTECTED REGION END -----*/	//	FastPS.h

/**
 *  FastPS class description:
 *    Caenels Fast Powersupply tango server
 */

namespace FastPS_ns
{
/*----- PROTECTED REGION ID(FastPS::Additional Class Declarations) ENABLED START -----*/

//	Additional Class Declarations

/*----- PROTECTED REGION END -----*/	//	FastPS::Additional Class Declarations

class FastPS : public TANGO_BASE_CLASS
{

/*----- PROTECTED REGION ID(FastPS::Data Members) ENABLED START -----*/

//	Add your own data members

public:
	struct sockaddr_in mysocket;
	int sock;
	omni_mutex *mutex;
	struct timeval last_state_time;
	struct timeval socket_state_time;
	std::map<int, std::string> error_mapping;
	int32_t wave_size, hw_wave_size;
	Tango::WAttribute *attr_set_current;	

/*----- PROTECTED REGION END -----*/	//	FastPS::Data Members

//	Device property data members
public:
	//	IpAddress:	
	string	ipAddress;
	//	TcpPort:	
	Tango::DevLong	tcpPort;
	//	TxRxSleep:	
	Tango::DevLong	txRxSleep;

//	Attribute data members
public:
	Tango::DevBoolean	*attr_CurrentControl_read;
	Tango::DevBoolean	*attr_VoltageControl_read;
	Tango::DevDouble	*attr_Voltage_read;
	Tango::DevDouble	*attr_Current_read;
	Tango::DevDouble	*attr_CurrentSet_read;
	Tango::DevBoolean	*attr_WaveformExternalTrigger_read;
	Tango::DevDouble	*attr_VoltageDCLink_read;
	Tango::DevDouble	*attr_MosfetTemperature_read;
	Tango::DevBoolean	*attr_AnalogControl_read;
	Tango::DevDouble	*attr_WaveformTimeScale_read;
	Tango::DevLong	*attr_WaveformPeriods_read;
	Tango::DevDouble	*attr_Waveform_read;
	Tango::DevDouble	*attr_HwWaveform_read;

//	Constructors and destructors
public:
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	FastPS(Tango::DeviceClass *cl,string &s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	FastPS(Tango::DeviceClass *cl,const char *s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device name
	 *	@param d	Device description.
	 */
	FastPS(Tango::DeviceClass *cl,const char *s,const char *d);
	/**
	 * The device object destructor.
	 */
	~FastPS() {delete_device();};


//	Miscellaneous methods
public:
	/*
	 *	will be called at device destruction or at init command.
	 */
	void delete_device();
	/*
	 *	Initialize the device
	 */
	virtual void init_device();
	/*
	 *	Read the device properties from database
	 */
	void get_device_property();
	/*
	 *	Always executed method before execution command method.
	 */
	virtual void always_executed_hook();


//	Attribute methods
public:
	//--------------------------------------------------------
	/*
	 *	Method      : FastPS::read_attr_hardware()
	 *	Description : Hardware acquisition for attributes.
	 */
	//--------------------------------------------------------
	virtual void read_attr_hardware(vector<long> &attr_list);
	//--------------------------------------------------------
	/*
	 *	Method      : FastPS::write_attr_hardware()
	 *	Description : Hardware writing for attributes.
	 */
	//--------------------------------------------------------
	virtual void write_attr_hardware(vector<long> &attr_list);

/**
 *	Attribute CurrentControl related methods
 *	Description: Enable current control mode
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_CurrentControl(Tango::Attribute &attr);
	virtual void write_CurrentControl(Tango::WAttribute &attr);
	virtual bool is_CurrentControl_allowed(Tango::AttReqType type);
/**
 *	Attribute VoltageControl related methods
 *	Description: Enable voltage control mode
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_VoltageControl(Tango::Attribute &attr);
	virtual void write_VoltageControl(Tango::WAttribute &attr);
	virtual bool is_VoltageControl_allowed(Tango::AttReqType type);
/**
 *	Attribute Voltage related methods
 *	Description: Read voltage
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_Voltage(Tango::Attribute &attr);
	virtual bool is_Voltage_allowed(Tango::AttReqType type);
/**
 *	Attribute Current related methods
 *	Description: Read current
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_Current(Tango::Attribute &attr);
	virtual void write_Current(Tango::WAttribute &attr);
	virtual bool is_Current_allowed(Tango::AttReqType type);
/**
 *	Attribute CurrentSet related methods
 *	Description: Read current last set
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_CurrentSet(Tango::Attribute &attr);
	virtual bool is_CurrentSet_allowed(Tango::AttReqType type);
/**
 *	Attribute WaveformExternalTrigger related methods
 *	Description: Enable external waveform trigger
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_WaveformExternalTrigger(Tango::Attribute &attr);
	virtual void write_WaveformExternalTrigger(Tango::WAttribute &attr);
	virtual bool is_WaveformExternalTrigger_allowed(Tango::AttReqType type);
/**
 *	Attribute VoltageDCLink related methods
 *	Description: Read voltage DC link
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_VoltageDCLink(Tango::Attribute &attr);
	virtual bool is_VoltageDCLink_allowed(Tango::AttReqType type);
/**
 *	Attribute MosfetTemperature related methods
 *	Description: Read mosfet heatsink temperature
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_MosfetTemperature(Tango::Attribute &attr);
	virtual bool is_MosfetTemperature_allowed(Tango::AttReqType type);
/**
 *	Attribute AnalogControl related methods
 *	Description: Enable external analog control
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_AnalogControl(Tango::Attribute &attr);
	virtual void write_AnalogControl(Tango::WAttribute &attr);
	virtual bool is_AnalogControl_allowed(Tango::AttReqType type);
/**
 *	Attribute WaveformTimeScale related methods
 *	Description: Set the timescale multiplication factor.
 *               By defaul tone waveform sample corresponds to 10us.
 *               This time (10us) is multiplicated by WaveformTimeScale factor.
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_WaveformTimeScale(Tango::Attribute &attr);
	virtual void write_WaveformTimeScale(Tango::WAttribute &attr);
	virtual bool is_WaveformTimeScale_allowed(Tango::AttReqType type);
/**
 *	Attribute WaveformPeriods related methods
 *	Description: Set waveform periodns. 0 means infinite loop
 *
 *	Data type:	Tango::DevLong
 *	Attr type:	Scalar
 */
	virtual void read_WaveformPeriods(Tango::Attribute &attr);
	virtual void write_WaveformPeriods(Tango::WAttribute &attr);
	virtual bool is_WaveformPeriods_allowed(Tango::AttReqType type);
/**
 *	Attribute Waveform related methods
 *	Description: Read/write a waveform (abs values) mitigated by TimeScale factor
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 500000
 */
	virtual void read_Waveform(Tango::Attribute &attr);
	virtual void write_Waveform(Tango::WAttribute &attr);
	virtual bool is_Waveform_allowed(Tango::AttReqType type);
/**
 *	Attribute HwWaveform related methods
 *	Description: Read/write a waveform directly from controller (no TimeScale postprocessing)
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 500000
 */
	virtual void read_HwWaveform(Tango::Attribute &attr);
	virtual void write_HwWaveform(Tango::WAttribute &attr);
	virtual bool is_HwWaveform_allowed(Tango::AttReqType type);


	//--------------------------------------------------------
	/**
	 *	Method      : FastPS::add_dynamic_attributes()
	 *	Description : Add dynamic attributes if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_attributes();




//	Command related methods
public:
	/**
	 *	Command On related method
	 *	Description: Switch power supply ON
	 *
	 */
	virtual void on();
	virtual bool is_On_allowed(const CORBA::Any &any);
	/**
	 *	Command Off related method
	 *	Description: Switch power supply OFF
	 *
	 */
	virtual void off();
	virtual bool is_Off_allowed(const CORBA::Any &any);
	/**
	 *	Command StartWaveform related method
	 *	Description: Start Waveform
	 *
	 */
	virtual void start_waveform();
	virtual bool is_StartWaveform_allowed(const CORBA::Any &any);
	/**
	 *	Command Reset related method
	 *	Description: Reset status errors
	 *
	 */
	virtual void reset();
	virtual bool is_Reset_allowed(const CORBA::Any &any);
	/**
	 *	Command HwReset related method
	 *	Description: Reboot the power supply
	 *
	 */
	virtual void hw_reset();
	virtual bool is_HwReset_allowed(const CORBA::Any &any);
	/**
	 *	Command StopWaveform related method
	 *	Description: Start Waveform
	 *
	 */
	virtual void stop_waveform();
	virtual bool is_StopWaveform_allowed(const CORBA::Any &any);


	//--------------------------------------------------------
	/**
	 *	Method      : FastPS::add_dynamic_commands()
	 *	Description : Add dynamic commands if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_commands();

/*----- PROTECTED REGION ID(FastPS::Additional Method prototypes) ENABLED START -----*/

//	Additional Method prototypes

	int sock_connect();
	void sock_flush(); 
	long write_cmd(char *in, char *out, int timeout);
	void map_errors(void);	
	void set_DevState(void);
	void read_waveform(double *waveform, int32_t *size); 
	void write_waveform(double *waveform, int32_t size, int32_t time_scale); 

/*----- PROTECTED REGION END -----*/	//	FastPS::Additional Method prototypes
};

/*----- PROTECTED REGION ID(FastPS::Additional Classes Definitions) ENABLED START -----*/

//	Additional Classes Definitions

/*----- PROTECTED REGION END -----*/	//	FastPS::Additional Classes Definitions

}	//	End of namespace

#endif   //	FastPS_H
