/*----- PROTECTED REGION ID(FastPSClass.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        FastPSClass.cpp
//
// description : C++ source for the FastPSClass.
//               A singleton class derived from DeviceClass.
//               It implements the command and attribute list
//               and all properties and methods required
//               by the FastPS once per process.
//
// project :     CaenElsFastPS
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
//
// Elettra-Sincrotrone Trieste S.C.p.A. di interesse nazionale
// Strada Statale 14 - km 163,5 in AREA Science Park
// 34149 Basovizza, Trieste ITALY
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <FastPSClass.h>

/*----- PROTECTED REGION END -----*/	//	FastPSClass.cpp

//-------------------------------------------------------------------
/**
 *	Create FastPSClass singleton and
 *	return it in a C function for Python usage
 */
//-------------------------------------------------------------------
extern "C" {
#ifdef _TG_WINDOWS_

__declspec(dllexport)

#endif

	Tango::DeviceClass *_create_FastPS_class(const char *name) {
		return FastPS_ns::FastPSClass::init(name);
	}
}

namespace FastPS_ns
{
//===================================================================
//	Initialize pointer for singleton pattern
//===================================================================
FastPSClass *FastPSClass::_instance = NULL;

//--------------------------------------------------------
/**
 * method : 		FastPSClass::FastPSClass(string &s)
 * description : 	constructor for the FastPSClass
 *
 * @param s	The class name
 */
//--------------------------------------------------------
FastPSClass::FastPSClass(string &s):Tango::DeviceClass(s)
{
	cout2 << "Entering FastPSClass constructor" << endl;
	set_default_property();
	write_class_property();

	/*----- PROTECTED REGION ID(FastPSClass::constructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::constructor

	cout2 << "Leaving FastPSClass constructor" << endl;
}

//--------------------------------------------------------
/**
 * method : 		FastPSClass::~FastPSClass()
 * description : 	destructor for the FastPSClass
 */
//--------------------------------------------------------
FastPSClass::~FastPSClass()
{
	/*----- PROTECTED REGION ID(FastPSClass::destructor) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::destructor

	_instance = NULL;
}


//--------------------------------------------------------
/**
 * method : 		FastPSClass::init
 * description : 	Create the object if not already done.
 *                  Otherwise, just return a pointer to the object
 *
 * @param	name	The class name
 */
//--------------------------------------------------------
FastPSClass *FastPSClass::init(const char *name)
{
	if (_instance == NULL)
	{
		try
		{
			string s(name);
			_instance = new FastPSClass(s);
		}
		catch (bad_alloc &)
		{
			throw;
		}
	}
	return _instance;
}

//--------------------------------------------------------
/**
 * method : 		FastPSClass::instance
 * description : 	Check if object already created,
 *                  and return a pointer to the object
 */
//--------------------------------------------------------
FastPSClass *FastPSClass::instance()
{
	if (_instance == NULL)
	{
		cerr << "Class is not initialised !!" << endl;
		exit(-1);
	}
	return _instance;
}



//===================================================================
//	Command execution method calls
//===================================================================
//--------------------------------------------------------
/**
 * method : 		OnClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *OnClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "OnClass::execute(): arrived" << endl;
	((static_cast<FastPS *>(device))->on());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		OffClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *OffClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "OffClass::execute(): arrived" << endl;
	((static_cast<FastPS *>(device))->off());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		StartWaveformClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *StartWaveformClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "StartWaveformClass::execute(): arrived" << endl;
	((static_cast<FastPS *>(device))->start_waveform());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		ResetClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *ResetClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "ResetClass::execute(): arrived" << endl;
	((static_cast<FastPS *>(device))->reset());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		HwResetClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *HwResetClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "HwResetClass::execute(): arrived" << endl;
	((static_cast<FastPS *>(device))->hw_reset());
	return new CORBA::Any();
}

//--------------------------------------------------------
/**
 * method : 		StopWaveformClass::execute()
 * description : 	method to trigger the execution of the command.
 *
 * @param	device	The device on which the command must be executed
 * @param	in_any	The command input data
 *
 *	returns The command output data (packed in the Any object)
 */
//--------------------------------------------------------
CORBA::Any *StopWaveformClass::execute(Tango::DeviceImpl *device, TANGO_UNUSED(const CORBA::Any &in_any))
{
	cout2 << "StopWaveformClass::execute(): arrived" << endl;
	((static_cast<FastPS *>(device))->stop_waveform());
	return new CORBA::Any();
}


//===================================================================
//	Properties management
//===================================================================
//--------------------------------------------------------
/**
 *	Method      : FastPSClass::get_class_property()
 *	Description : Get the class property for specified name.
 */
//--------------------------------------------------------
Tango::DbDatum FastPSClass::get_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_prop.size() ; i++)
		if (cl_prop[i].name == prop_name)
			return cl_prop[i];
	//	if not found, returns  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : FastPSClass::get_default_device_property()
 *	Description : Return the default value for device property.
 */
//--------------------------------------------------------
Tango::DbDatum FastPSClass::get_default_device_property(string &prop_name)
{
	for (unsigned int i=0 ; i<dev_def_prop.size() ; i++)
		if (dev_def_prop[i].name == prop_name)
			return dev_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}

//--------------------------------------------------------
/**
 *	Method      : FastPSClass::get_default_class_property()
 *	Description : Return the default value for class property.
 */
//--------------------------------------------------------
Tango::DbDatum FastPSClass::get_default_class_property(string &prop_name)
{
	for (unsigned int i=0 ; i<cl_def_prop.size() ; i++)
		if (cl_def_prop[i].name == prop_name)
			return cl_def_prop[i];
	//	if not found, return  an empty DbDatum
	return Tango::DbDatum(prop_name);
}


//--------------------------------------------------------
/**
 *	Method      : FastPSClass::set_default_property()
 *	Description : Set default property (class and device) for wizard.
 *                For each property, add to wizard property name and description.
 *                If default value has been set, add it to wizard property and
 *                store it in a DbDatum.
 */
//--------------------------------------------------------
void FastPSClass::set_default_property()
{
	string	prop_name;
	string	prop_desc;
	string	prop_def;
	vector<string>	vect_data;

	//	Set Default Class Properties

	//	Set Default device Properties
	prop_name = "IpAddress";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "TcpPort";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
	prop_name = "TxRxSleep";
	prop_desc = "";
	prop_def  = "";
	vect_data.clear();
	if (prop_def.length()>0)
	{
		Tango::DbDatum	data(prop_name);
		data << vect_data ;
		dev_def_prop.push_back(data);
		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
	}
	else
		add_wiz_dev_prop(prop_name, prop_desc);
}

//--------------------------------------------------------
/**
 *	Method      : FastPSClass::write_class_property()
 *	Description : Set class description fields as property in database
 */
//--------------------------------------------------------
void FastPSClass::write_class_property()
{
	//	First time, check if database used
	if (Tango::Util::_UseDb == false)
		return;

	Tango::DbData	data;
	string	classname = get_name();
	string	header;
	string::size_type	start, end;

	//	Put title
	Tango::DbDatum	title("ProjectTitle");
	string	str_title("CaenElsFastPS");
	title << str_title;
	data.push_back(title);

	//	Put Description
	Tango::DbDatum	description("Description");
	vector<string>	str_desc;
	str_desc.push_back("Caenels Fast Powersupply tango server");
	description << str_desc;
	data.push_back(description);

	//  Put inheritance
	Tango::DbDatum	inher_datum("InheritedFrom");
	vector<string> inheritance;
	inheritance.push_back("TANGO_BASE_CLASS");
	inher_datum << inheritance;
	data.push_back(inher_datum);

	//	Call database and and values
	get_db_class()->put_property(data);
}

//===================================================================
//	Factory methods
//===================================================================

//--------------------------------------------------------
/**
 *	Method      : FastPSClass::device_factory()
 *	Description : Create the device object(s)
 *                and store them in the device list
 */
//--------------------------------------------------------
void FastPSClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
{
	/*----- PROTECTED REGION ID(FastPSClass::device_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::device_factory_before

	//	Create devices and add it into the device list
	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		cout4 << "Device name : " << (*devlist_ptr)[i].in() << endl;
		device_list.push_back(new FastPS(this, (*devlist_ptr)[i]));
	}

	//	Manage dynamic attributes if any
	erase_dynamic_attributes(devlist_ptr, get_class_attr()->get_attr_list());

	//	Export devices to the outside world
	for (unsigned long i=1 ; i<=devlist_ptr->length() ; i++)
	{
		//	Add dynamic attributes if any
		FastPS *dev = static_cast<FastPS *>(device_list[device_list.size()-i]);
		dev->add_dynamic_attributes();

		//	Check before if database used.
		if ((Tango::Util::_UseDb == true) && (Tango::Util::_FileDb == false))
			export_device(dev);
		else
			export_device(dev, dev->get_name().c_str());
	}

	/*----- PROTECTED REGION ID(FastPSClass::device_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::device_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : FastPSClass::attribute_factory()
 *	Description : Create the attribute object(s)
 *                and store them in the attribute list
 */
//--------------------------------------------------------
void FastPSClass::attribute_factory(vector<Tango::Attr *> &att_list)
{
	/*----- PROTECTED REGION ID(FastPSClass::attribute_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::attribute_factory_before
	//	Attribute : CurrentControl
	CurrentControlAttrib	*currentcontrol = new CurrentControlAttrib();
	Tango::UserDefaultAttrProp	currentcontrol_prop;
	currentcontrol_prop.set_description("Enable current control mode");
	//	label	not set for CurrentControl
	//	unit	not set for CurrentControl
	//	standard_unit	not set for CurrentControl
	//	display_unit	not set for CurrentControl
	//	format	not set for CurrentControl
	//	max_value	not set for CurrentControl
	//	min_value	not set for CurrentControl
	//	max_alarm	not set for CurrentControl
	//	min_alarm	not set for CurrentControl
	//	max_warning	not set for CurrentControl
	//	min_warning	not set for CurrentControl
	//	delta_t	not set for CurrentControl
	//	delta_val	not set for CurrentControl
	
	currentcontrol->set_default_properties(currentcontrol_prop);
	//	Not Polled
	currentcontrol->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(currentcontrol);

	//	Attribute : VoltageControl
	VoltageControlAttrib	*voltagecontrol = new VoltageControlAttrib();
	Tango::UserDefaultAttrProp	voltagecontrol_prop;
	voltagecontrol_prop.set_description("Enable voltage control mode");
	//	label	not set for VoltageControl
	//	unit	not set for VoltageControl
	//	standard_unit	not set for VoltageControl
	//	display_unit	not set for VoltageControl
	//	format	not set for VoltageControl
	//	max_value	not set for VoltageControl
	//	min_value	not set for VoltageControl
	//	max_alarm	not set for VoltageControl
	//	min_alarm	not set for VoltageControl
	//	max_warning	not set for VoltageControl
	//	min_warning	not set for VoltageControl
	//	delta_t	not set for VoltageControl
	//	delta_val	not set for VoltageControl
	
	voltagecontrol->set_default_properties(voltagecontrol_prop);
	//	Not Polled
	voltagecontrol->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(voltagecontrol);

	//	Attribute : Voltage
	VoltageAttrib	*voltage = new VoltageAttrib();
	Tango::UserDefaultAttrProp	voltage_prop;
	voltage_prop.set_description("Read voltage");
	//	label	not set for Voltage
	voltage_prop.set_unit("V");
	//	standard_unit	not set for Voltage
	voltage_prop.set_display_unit("V");
	voltage_prop.set_format("%.3f");
	voltage_prop.set_max_value("1000");
	voltage_prop.set_min_value("-1000");
	//	max_alarm	not set for Voltage
	//	min_alarm	not set for Voltage
	//	max_warning	not set for Voltage
	//	min_warning	not set for Voltage
	//	delta_t	not set for Voltage
	//	delta_val	not set for Voltage
	
	voltage->set_default_properties(voltage_prop);
	//	Not Polled
	voltage->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(voltage);

	//	Attribute : Current
	CurrentAttrib	*current = new CurrentAttrib();
	Tango::UserDefaultAttrProp	current_prop;
	current_prop.set_description("Read current");
	//	label	not set for Current
	current_prop.set_unit("A");
	//	standard_unit	not set for Current
	current_prop.set_display_unit("A");
	current_prop.set_format("%.4f");
	current_prop.set_max_value("1000");
	current_prop.set_min_value("-1000");
	//	max_alarm	not set for Current
	//	min_alarm	not set for Current
	//	max_warning	not set for Current
	//	min_warning	not set for Current
	//	delta_t	not set for Current
	//	delta_val	not set for Current
	
	current->set_default_properties(current_prop);
	//	Not Polled
	current->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(current);

	//	Attribute : CurrentSet
	CurrentSetAttrib	*currentset = new CurrentSetAttrib();
	Tango::UserDefaultAttrProp	currentset_prop;
	currentset_prop.set_description("Read current last set");
	//	label	not set for CurrentSet
	currentset_prop.set_unit("A");
	//	standard_unit	not set for CurrentSet
	currentset_prop.set_display_unit("A");
	currentset_prop.set_format("%.4f");
	currentset_prop.set_max_value("1000");
	currentset_prop.set_min_value("-1000");
	//	max_alarm	not set for CurrentSet
	//	min_alarm	not set for CurrentSet
	//	max_warning	not set for CurrentSet
	//	min_warning	not set for CurrentSet
	//	delta_t	not set for CurrentSet
	//	delta_val	not set for CurrentSet
	
	currentset->set_default_properties(currentset_prop);
	//	Not Polled
	currentset->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(currentset);

	//	Attribute : WaveformExternalTrigger
	WaveformExternalTriggerAttrib	*waveformexternaltrigger = new WaveformExternalTriggerAttrib();
	Tango::UserDefaultAttrProp	waveformexternaltrigger_prop;
	waveformexternaltrigger_prop.set_description("Enable external waveform trigger");
	//	label	not set for WaveformExternalTrigger
	//	unit	not set for WaveformExternalTrigger
	//	standard_unit	not set for WaveformExternalTrigger
	//	display_unit	not set for WaveformExternalTrigger
	//	format	not set for WaveformExternalTrigger
	//	max_value	not set for WaveformExternalTrigger
	//	min_value	not set for WaveformExternalTrigger
	//	max_alarm	not set for WaveformExternalTrigger
	//	min_alarm	not set for WaveformExternalTrigger
	//	max_warning	not set for WaveformExternalTrigger
	//	min_warning	not set for WaveformExternalTrigger
	//	delta_t	not set for WaveformExternalTrigger
	//	delta_val	not set for WaveformExternalTrigger
	
	waveformexternaltrigger->set_default_properties(waveformexternaltrigger_prop);
	//	Not Polled
	waveformexternaltrigger->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(waveformexternaltrigger);

	//	Attribute : VoltageDCLink
	VoltageDCLinkAttrib	*voltagedclink = new VoltageDCLinkAttrib();
	Tango::UserDefaultAttrProp	voltagedclink_prop;
	voltagedclink_prop.set_description("Read voltage DC link");
	//	label	not set for VoltageDCLink
	voltagedclink_prop.set_unit("V");
	//	standard_unit	not set for VoltageDCLink
	voltagedclink_prop.set_display_unit("V");
	voltagedclink_prop.set_format("%.3f");
	voltagedclink_prop.set_max_value("1000");
	voltagedclink_prop.set_min_value("-1000");
	//	max_alarm	not set for VoltageDCLink
	//	min_alarm	not set for VoltageDCLink
	//	max_warning	not set for VoltageDCLink
	//	min_warning	not set for VoltageDCLink
	//	delta_t	not set for VoltageDCLink
	//	delta_val	not set for VoltageDCLink
	
	voltagedclink->set_default_properties(voltagedclink_prop);
	//	Not Polled
	voltagedclink->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(voltagedclink);

	//	Attribute : MosfetTemperature
	MosfetTemperatureAttrib	*mosfettemperature = new MosfetTemperatureAttrib();
	Tango::UserDefaultAttrProp	mosfettemperature_prop;
	mosfettemperature_prop.set_description("Read mosfet heatsink temperature");
	//	label	not set for MosfetTemperature
	mosfettemperature_prop.set_unit("C");
	//	standard_unit	not set for MosfetTemperature
	mosfettemperature_prop.set_display_unit("C");
	mosfettemperature_prop.set_format("%.1f");
	mosfettemperature_prop.set_max_value("300");
	mosfettemperature_prop.set_min_value("0");
	//	max_alarm	not set for MosfetTemperature
	//	min_alarm	not set for MosfetTemperature
	//	max_warning	not set for MosfetTemperature
	//	min_warning	not set for MosfetTemperature
	//	delta_t	not set for MosfetTemperature
	//	delta_val	not set for MosfetTemperature
	
	mosfettemperature->set_default_properties(mosfettemperature_prop);
	//	Not Polled
	mosfettemperature->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(mosfettemperature);

	//	Attribute : AnalogControl
	AnalogControlAttrib	*analogcontrol = new AnalogControlAttrib();
	Tango::UserDefaultAttrProp	analogcontrol_prop;
	analogcontrol_prop.set_description("Enable external analog control");
	//	label	not set for AnalogControl
	//	unit	not set for AnalogControl
	//	standard_unit	not set for AnalogControl
	//	display_unit	not set for AnalogControl
	//	format	not set for AnalogControl
	//	max_value	not set for AnalogControl
	//	min_value	not set for AnalogControl
	//	max_alarm	not set for AnalogControl
	//	min_alarm	not set for AnalogControl
	//	max_warning	not set for AnalogControl
	//	min_warning	not set for AnalogControl
	//	delta_t	not set for AnalogControl
	//	delta_val	not set for AnalogControl
	
	analogcontrol->set_default_properties(analogcontrol_prop);
	//	Not Polled
	analogcontrol->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(analogcontrol);

	//	Attribute : WaveformTimeScale
	WaveformTimeScaleAttrib	*waveformtimescale = new WaveformTimeScaleAttrib();
	Tango::UserDefaultAttrProp	waveformtimescale_prop;
	waveformtimescale_prop.set_description("Set the timescale multiplication factor.\nBy defaul tone waveform sample corresponds to 10us.\nThis time (10us) is multiplicated by WaveformTimeScale factor.");
	//	label	not set for WaveformTimeScale
	//	unit	not set for WaveformTimeScale
	//	standard_unit	not set for WaveformTimeScale
	//	display_unit	not set for WaveformTimeScale
	//	format	not set for WaveformTimeScale
	//	max_value	not set for WaveformTimeScale
	//	min_value	not set for WaveformTimeScale
	//	max_alarm	not set for WaveformTimeScale
	//	min_alarm	not set for WaveformTimeScale
	//	max_warning	not set for WaveformTimeScale
	//	min_warning	not set for WaveformTimeScale
	//	delta_t	not set for WaveformTimeScale
	//	delta_val	not set for WaveformTimeScale
	
	waveformtimescale->set_default_properties(waveformtimescale_prop);
	//	Not Polled
	waveformtimescale->set_disp_level(Tango::OPERATOR);
	waveformtimescale->set_memorized();
	waveformtimescale->set_memorized_init(true);
	att_list.push_back(waveformtimescale);

	//	Attribute : WaveformPeriods
	WaveformPeriodsAttrib	*waveformperiods = new WaveformPeriodsAttrib();
	Tango::UserDefaultAttrProp	waveformperiods_prop;
	waveformperiods_prop.set_description("Set waveform periodns. 0 means infinite loop");
	//	label	not set for WaveformPeriods
	//	unit	not set for WaveformPeriods
	//	standard_unit	not set for WaveformPeriods
	//	display_unit	not set for WaveformPeriods
	waveformperiods_prop.set_format("%d");
	waveformperiods_prop.set_max_value("2000000000");
	waveformperiods_prop.set_min_value("0");
	//	max_alarm	not set for WaveformPeriods
	//	min_alarm	not set for WaveformPeriods
	//	max_warning	not set for WaveformPeriods
	//	min_warning	not set for WaveformPeriods
	//	delta_t	not set for WaveformPeriods
	//	delta_val	not set for WaveformPeriods
	
	waveformperiods->set_default_properties(waveformperiods_prop);
	//	Not Polled
	waveformperiods->set_disp_level(Tango::OPERATOR);
	waveformperiods->set_memorized();
	waveformperiods->set_memorized_init(true);
	att_list.push_back(waveformperiods);

	//	Attribute : Waveform
	WaveformAttrib	*waveform = new WaveformAttrib();
	Tango::UserDefaultAttrProp	waveform_prop;
	waveform_prop.set_description("Read/write a waveform (abs values) mitigated by TimeScale factor");
	//	label	not set for Waveform
	//	unit	not set for Waveform
	//	standard_unit	not set for Waveform
	//	display_unit	not set for Waveform
	waveform_prop.set_format("%.4f");
	waveform_prop.set_max_value("1000");
	waveform_prop.set_min_value("-1000");
	//	max_alarm	not set for Waveform
	//	min_alarm	not set for Waveform
	//	max_warning	not set for Waveform
	//	min_warning	not set for Waveform
	//	delta_t	not set for Waveform
	//	delta_val	not set for Waveform
	
	waveform->set_default_properties(waveform_prop);
	//	Not Polled
	waveform->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(waveform);

	//	Attribute : HwWaveform
	HwWaveformAttrib	*hwwaveform = new HwWaveformAttrib();
	Tango::UserDefaultAttrProp	hwwaveform_prop;
	hwwaveform_prop.set_description("Read/write a waveform directly from controller (no TimeScale postprocessing)");
	//	label	not set for HwWaveform
	hwwaveform_prop.set_unit("A");
	//	standard_unit	not set for HwWaveform
	hwwaveform_prop.set_display_unit("A");
	hwwaveform_prop.set_format("%.4f");
	hwwaveform_prop.set_max_value("1000");
	hwwaveform_prop.set_min_value("-1000");
	//	max_alarm	not set for HwWaveform
	//	min_alarm	not set for HwWaveform
	//	max_warning	not set for HwWaveform
	//	min_warning	not set for HwWaveform
	//	delta_t	not set for HwWaveform
	//	delta_val	not set for HwWaveform
	
	hwwaveform->set_default_properties(hwwaveform_prop);
	//	Not Polled
	hwwaveform->set_disp_level(Tango::OPERATOR);
	//	Not Memorized
	att_list.push_back(hwwaveform);


	//	Create a list of static attributes
	create_static_attribute_list(get_class_attr()->get_attr_list());
	/*----- PROTECTED REGION ID(FastPSClass::attribute_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::attribute_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : FastPSClass::pipe_factory()
 *	Description : Create the pipe object(s)
 *                and store them in the pipe list
 */
//--------------------------------------------------------
void FastPSClass::pipe_factory()
{
	/*----- PROTECTED REGION ID(FastPSClass::pipe_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::pipe_factory_before
	/*----- PROTECTED REGION ID(FastPSClass::pipe_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::pipe_factory_after
}
//--------------------------------------------------------
/**
 *	Method      : FastPSClass::command_factory()
 *	Description : Create the command object(s)
 *                and store them in the command list
 */
//--------------------------------------------------------
void FastPSClass::command_factory()
{
	/*----- PROTECTED REGION ID(FastPSClass::command_factory_before) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::command_factory_before


	//	Command On
	OnClass	*pOnCmd =
		new OnClass("On",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pOnCmd);

	//	Command Off
	OffClass	*pOffCmd =
		new OffClass("Off",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pOffCmd);

	//	Command StartWaveform
	StartWaveformClass	*pStartWaveformCmd =
		new StartWaveformClass("StartWaveform",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pStartWaveformCmd);

	//	Command Reset
	ResetClass	*pResetCmd =
		new ResetClass("Reset",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pResetCmd);

	//	Command HwReset
	HwResetClass	*pHwResetCmd =
		new HwResetClass("HwReset",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pHwResetCmd);

	//	Command StopWaveform
	StopWaveformClass	*pStopWaveformCmd =
		new StopWaveformClass("StopWaveform",
			Tango::DEV_VOID, Tango::DEV_VOID,
			"",
			"",
			Tango::OPERATOR);
	command_list.push_back(pStopWaveformCmd);

	/*----- PROTECTED REGION ID(FastPSClass::command_factory_after) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::command_factory_after
}

//===================================================================
//	Dynamic attributes related methods
//===================================================================

//--------------------------------------------------------
/**
 * method : 		FastPSClass::create_static_attribute_list
 * description : 	Create the a list of static attributes
 *
 * @param	att_list	the ceated attribute list
 */
//--------------------------------------------------------
void FastPSClass::create_static_attribute_list(vector<Tango::Attr *> &att_list)
{
	for (unsigned long i=0 ; i<att_list.size() ; i++)
	{
		string att_name(att_list[i]->get_name());
		transform(att_name.begin(), att_name.end(), att_name.begin(), ::tolower);
		defaultAttList.push_back(att_name);
	}

	cout2 << defaultAttList.size() << " attributes in default list" << endl;

	/*----- PROTECTED REGION ID(FastPSClass::create_static_att_list) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::create_static_att_list
}


//--------------------------------------------------------
/**
 * method : 		FastPSClass::erase_dynamic_attributes
 * description : 	delete the dynamic attributes if any.
 *
 * @param	devlist_ptr	the device list pointer
 * @param	list of all attributes
 */
//--------------------------------------------------------
void FastPSClass::erase_dynamic_attributes(const Tango::DevVarStringArray *devlist_ptr, vector<Tango::Attr *> &att_list)
{
	Tango::Util *tg = Tango::Util::instance();

	for (unsigned long i=0 ; i<devlist_ptr->length() ; i++)
	{
		Tango::DeviceImpl *dev_impl = tg->get_device_by_name(((string)(*devlist_ptr)[i]).c_str());
		FastPS *dev = static_cast<FastPS *> (dev_impl);

		vector<Tango::Attribute *> &dev_att_list = dev->get_device_attr()->get_attribute_list();
		vector<Tango::Attribute *>::iterator ite_att;
		for (ite_att=dev_att_list.begin() ; ite_att != dev_att_list.end() ; ++ite_att)
		{
			string att_name((*ite_att)->get_name_lower());
			if ((att_name == "state") || (att_name == "status"))
				continue;
			vector<string>::iterator ite_str = find(defaultAttList.begin(), defaultAttList.end(), att_name);
			if (ite_str == defaultAttList.end())
			{
				cout2 << att_name << " is a UNWANTED dynamic attribute for device " << (*devlist_ptr)[i] << endl;
				Tango::Attribute &att = dev->get_device_attr()->get_attr_by_name(att_name.c_str());
				dev->remove_attribute(att_list[att.get_attr_idx()], true, false);
				--ite_att;
			}
		}
	}
	/*----- PROTECTED REGION ID(FastPSClass::erase_dynamic_attributes) ENABLED START -----*/
	
	/*----- PROTECTED REGION END -----*/	//	FastPSClass::erase_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Method      : FastPSClass::get_attr_by_name()
 *	Description : returns Tango::Attr * object found by name
 */
//--------------------------------------------------------
Tango::Attr *FastPSClass::get_attr_object_by_name(vector<Tango::Attr *> &att_list, string attname)
{
	vector<Tango::Attr *>::iterator it;
	for (it=att_list.begin() ; it<att_list.end() ; ++it)
		if ((*it)->get_name()==attname)
			return (*it);
	//	Attr does not exist
	return NULL;
}


/*----- PROTECTED REGION ID(FastPSClass::Additional Methods) ENABLED START -----*/

/*----- PROTECTED REGION END -----*/	//	FastPSClass::Additional Methods
} //	namespace
